/*
 * wsi603a.c
 *
 *  Created on: 17-Jun-2009
 *      Author: sjm
 */

/* Read DS2760 (family code 0x30) ADC */

#include <stdio.h>
#include <math.h>
#include "werr.h"
//#include "mlan.h"
#include "ownet.h"
#include "applctn.h"
#include "devices.h"
#include "wsi603a.h"
#include "setup.h"
#include "intl.h"

#define WSI603A_TRIES 4
#define CRAZYSPEED 250.0F

int crc_ok(uchar *block, int length)
{
  int i, sum=0;
  for (i=0; i<length; ++i)
	sum += block[i];

  sum &= 0xff;

//  if (sum != block[length])
//  {
//	for (i=0; i<=length; ++i)
//	  printf("%02x ", block[i]);
//	printf("%02x \n", sum);
//  }

  return ((sum&0xff) == block[length]);
}

/*
* Read a block from the DS2670
*
*/
int wsi603a_read_once(int device,  wsi603a_struct *result)
{
  uchar block[9];
  int i;

  if (!devices_access(device))
    return -1;

  block[0] = 0x69; /* Read Data */
  block[1] = 0x88; /* Start of SRAM */
  for (i=2; i<=8; block[i++]=0xff);

  if(owBlock(devices_portnum(device), FALSE, block, 9))
  {
    int value;

    // Check the crc
    if (!crc_ok(&block[2], 6))
      return -1;

    result->windSpeed = 1.6294F * (float) block[3]; // Result in mph - Ignore cal for now

    if (result->windSpeed >= CRAZYSPEED)
      return -1;

    result->windPoint = (int) block[4]; // Result is 1 - 16

    if ((block[4] & 0xf0) != 0) return -1;

    result->intensity = (float) block[6];

//    printf("sp = %f, pt = %d, int = %f", result->windSpeed, result->windPoint, result->intensity);

    if (!devices_access(device))
      return -1;

    block[0] = 0x69; /* Read Data */
    block[1] = 0x18; /* Start of temperature data */
    for (i=2; i<4; block[i++]=0xff);

    if (owBlock(devices_portnum(device), FALSE, block, 4))
    {
      value = (block[2] << 3) | (block[3] >> 5);
      if (value & 0x400) // Check sign bit
        value |= ~0x7ff; // Set all bits above bit 10
      result->T = 0.125F * (float) value;

//      printf(", T = %f\n", result->T);

      return 0; /* Ok */
    }

    return 0; /* Ok */
  }

  return -1 ; /* Failed */
}

int wsi603a_read(int device,  wsi603a_struct *result)
{
  int i;

  for (i=0; i<WSI603A_TRIES; ++i)
  {
	if (0==wsi603a_read_once(device, result))
	  return 0;
	msDelay(50);
  }

  werr(WERR_DEBUG0, "Too many retries on WSI603A");
  return -1;
}
